/*
 * tcpWin.h
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#ifndef T2_TCPWIN_H_INCLUDED
#define T2_TCPWIN_H_INCLUDED

// Global includes


// Local includes

#include "t2Plugin.h"


/* ========================================================================== */
/* ------------------------ USER CONFIGURATION FLAGS ------------------------ */
/* ========================================================================== */

#define TCPWIN_THRES      1 // TCP window size threshold undershoot flag
#define TCPWIN_FLWPRG     2 // Threshold to abort a flow
#define TCPWIN_MINPKTS   50 // Summary file: minimal TCP packets seen to start saving process
#define TCPWIN_MAXWSCNT 100 // Summary file: maximal number of window size threshold count array elements

#define TCPWIN_SUFFIX "_tcpwin.txt"  // Summary file: file name suffix

/* ========================================================================== */
/* ------------------------- DO NOT EDIT BELOW HERE ------------------------- */
/* ========================================================================== */


// plugin defines

// tcpWinStat status variable
// 0x01: FIN
// 0x02: SYN
// 0x04: RST
// 0x08: PSH
// 0x10: ACK
#define TCPWIN_STAT_THU   0x40 // TCP window size threshold undershoot
#define TCPWIN_STAT_ABNRM 0x80

// tcpWinState
#define TCPWIN_STATE_NEW  0x00 // <--
#define TCPWIN_STATE_BGN  0x01 // <--
#define TCPWIN_STATE_EST  0x02 // <--
#define TCPWIN_STATE_ABRT 0x04 // <--

// timeouts @ states
#define TCPWIN_TMT_NEW  10.0f   // <-- 10 s
#define TCPWIN_TMT_BGN   2.2f   // <-- 2.2 s
#define TCPWIN_TMT_EST   1.0f   // <-- 1.0 s
#define TCPWIN_TMT_ABRT  0.001f // <-- 1 ms


// Structure

typedef struct {
    ipAddr_t wzip  [TCPWIN_MAXWSCNT]; // IP address array
#if SUBNET_ON != 0 && (AGGREGATIONFLAG & SUBNET) == 0
    uint32_t sID   [TCPWIN_MAXWSCNT]; // subnetID
#endif // SUBNET_ON != 0 && (AGGREGATIONFLAG & SUBNET) == 0
    uint32_t tcpCnt[TCPWIN_MAXWSCNT]; // TCP packet count
    float    wzCnt [TCPWIN_MAXWSCNT]; // window size count
    int      wzi;                     // window size index
} gwz_t;


// Plugin structure

typedef struct { // always large variables first to limit memory fragmentation
    uint32_t tcpWinInit; // initial window size
    uint32_t winThCnt;   // win undershoot count
    uint8_t  ttl;        // TTL
    uint8_t  stat;       // plugin flow status
    uint8_t  state;      // plugin flow state
} tcpWinFlow_t;


// plugin struct pointer for potential dependencies
extern tcpWinFlow_t *tcpWinFlows;
extern gwz_t *gwzP;

#endif // T2_TCPWIN_H_INCLUDED
